@@ -279,6 +279,27 @@ static gboolean bee_irc_user_typing( bee_t *bee, bee_user_t *bu, uint32_t flags
 	return TRUE;
 }
 
+static gboolean bee_irc_user_action_response( bee_t *bee, bee_user_t *bu, const char *action, char * const args[], void *data )
+{
+	irc_t *irc = (irc_t *) bee->ui_data;
+	GString *msg = g_string_new( "\001" );
+	
+	g_string_append( msg, action );
+	while( *args )
+	{
+		if( strchr( *args, ' ' ) )
+			g_string_append_printf( msg, " \"%s\"", *args );
+		else
+			g_string_append_printf( msg, " %s", *args );
+		args ++;
+	}
+	g_string_append_c( msg, '\001' );
+	
+	irc_send_msg( (irc_user_t *) bu->ui_data, "NOTICE", irc->user->nick, msg->str, NULL );
+	
+	return TRUE;
+}
+
 static gboolean bee_irc_user_nick_update( irc_user_t *iu );
 
 static gboolean bee_irc_user_fullname( bee_t *bee, bee_user_t *bu )
@@ -362,6 +383,24 @@ static gboolean bee_irc_user_nick_update( irc_user_t *iu )
 	return TRUE;
 }
 
+void bee_irc_user_nick_reset( irc_user_t *iu )
+{
+	bee_user_t *bu = iu->bu;
+	bee_user_flags_t online;
+	
+	if( bu == FALSE )
+		return;
+	
+	/* In this case, pretend the user is offline. */
+	if( ( online = bu->flags & BEE_USER_ONLINE ) )
+		bu->flags &= ~BEE_USER_ONLINE;
+	
+	nick_del( bu );
+	bee_irc_user_nick_update( iu );
+	
+	bu->flags |= online;
+}
+
 /* IRC->IM calls */
 
 static gboolean bee_irc_user_privmsg_cb( gpointer data, gint fd, b_input_condition cond );
@@ -473,6 +512,30 @@ static gboolean bee_irc_user_ctcp( irc_user_t *iu, char *const *ctcp )
 			return TRUE;
 		}
 	}
+	else if( g_strcasecmp( ctcp[0], "HELP" ) == 0 && iu->bu )
+	{
+		GString *supp = g_string_new( "Supported CTCPs:" );
+		GList *l;
+		
+		if( iu->bu->ic && iu->bu->ic->acc->prpl->transfer_request )
+			g_string_append( supp, " DCC SEND," );
+		if( iu->bu->ic && iu->bu->ic->acc->prpl->send_typing )
+			g_string_append( supp, " TYPING," );
+		if( iu->bu->ic->acc->prpl->buddy_action_list )
+			for( l = iu->bu->ic->acc->prpl->buddy_action_list( iu->bu ); l; l = l->next )
+			{
+				struct buddy_action *ba = l->data;
+				g_string_append_printf( supp, " %s (%s),",
+				                        ba->name, ba->description );
+			}
+		g_string_truncate( supp, supp->len - 1 );
+		irc_send_msg_f( iu, "NOTICE", iu->irc->user->nick, "\001HELP %s\001", supp->str );
+		g_string_free( supp, TRUE );
+	}
+	else if( iu->bu && iu->bu->ic && iu->bu->ic->acc->prpl->buddy_action )
+	{
+		iu->bu->ic->acc->prpl->buddy_action( iu->bu, ctcp[0], ctcp + 1, NULL );
+	}
 	
 	return FALSE;
 }
@@ -536,6 +599,7 @@ static gboolean bee_irc_chat_free( bee_t *bee, struct groupchat *c )
 		irc_channel_printf( ic, "Cleaning up channel, bye!" );
 	
 	ic->data = NULL;
+	c->ui_data = NULL;
 	irc_channel_del_user( ic, ic->irc->user, IRC_CDU_KICK, "Chatroom closed by server" );
 	
 	return TRUE;
@@ -793,7 +857,8 @@ static gboolean bee_irc_channel_chat_privmsg_cb( gpointer data, gint fd, b_input
 {
 	irc_channel_t *ic = data;
 	
-	bee_chat_msg( ic->irc->b, ic->data, ic->pastebuf->str, 0 );
+	if( ic->data )
+		bee_chat_msg( ic->irc->b, ic->data, ic->pastebuf->str, 0 );
 	
 	g_string_free( ic->pastebuf, TRUE );
 	ic->pastebuf = 0;
@@ -840,6 +905,11 @@ static gboolean bee_irc_channel_chat_part( irc_channel_t *ic, const char *msg )
 	if( c && c->ic->acc->prpl->chat_leave )
 		c->ic->acc->prpl->chat_leave( c );
 	
+	/* Remove references in both directions now. We don't need each other anymore. */
+	ic->data = NULL;
+	if( c )
+		c->ui_data = NULL;
+	
 	return TRUE;
 }
 
@@ -1019,6 +1089,7 @@ const struct bee_ui_funcs irc_ui_funcs = {
 	bee_irc_user_status,
 	bee_irc_user_msg,
 	bee_irc_user_typing,
+	bee_irc_user_action_response,
 	
 	bee_irc_chat_new,
 	bee_irc_chat_free,
