@@ -156,6 +156,7 @@ void strip_html( char *in )
 	char out[strlen(in)+1];
 	char *s = out, *cs;
 	int i, matched;
+	int taglen;
 	
 	memset( out, 0, sizeof( out ) );
 	
@@ -172,9 +173,18 @@ void strip_html( char *in )
 			while( *in && *in != '>' )
 				in ++;
 			
+			taglen = in - cs - 1;   /* not <0 because the above loop runs at least once */
 			if( *in )
 			{
-				if( g_strncasecmp( cs+1, "br", 2) == 0 )
+				if( g_strncasecmp( cs+1, "b", taglen) == 0 )
+					*(s++) = '\x02';
+				else if( g_strncasecmp( cs+1, "/b", taglen) == 0 )
+					*(s++) = '\x02';
+				else if( g_strncasecmp( cs+1, "i", taglen) == 0 )
+					*(s++) = '\x1f';
+				else if( g_strncasecmp( cs+1, "/i", taglen) == 0 )
+					*(s++) = '\x1f';
+				else if( g_strncasecmp( cs+1, "br", taglen) == 0 )
 					*(s++) = '\n';
 				in ++;
 			}
@@ -297,11 +307,10 @@ void http_decode( char *s )
 /* This fuction is safe, but make sure you call it safely as well! */
 void http_encode( char *s )
 {
-	char *t;
+	char t[strlen(s)+1];
 	int i, j;
 	
-	t = g_strdup( s );
-	
+	strcpy( t, s );
 	for( i = j = 0; t[i]; i ++, j ++ )
 	{
 		/* Warning: isalnum() is locale-aware, so don't use it here! */
@@ -319,8 +328,6 @@ void http_encode( char *s )
 		}
 	}
 	s[j] = 0;
-	
-	g_free( t );
 }
 
 /* Strip newlines from a string. Modifies the string passed to it. */ 
@@ -507,16 +514,17 @@ int bool2int( char *value )
 	return 0;
 }
 
-struct ns_srv_reply *srv_lookup( char *service, char *protocol, char *domain )
+struct ns_srv_reply **srv_lookup( char *service, char *protocol, char *domain )
 {	
-	struct ns_srv_reply *reply = NULL;
+	struct ns_srv_reply **replies = NULL;
 #ifdef HAVE_RESOLV_A
+	struct ns_srv_reply *reply = NULL;
 	char name[1024];
 	unsigned char querybuf[1024];
 	const unsigned char *buf;
 	ns_msg nsh;
 	ns_rr rr;
-	int i, len, size;
+	int i, n, len, size;
 	
 	g_snprintf( name, sizeof( name ), "_%s._%s.%s", service, protocol, domain );
 	
@@ -526,37 +534,56 @@ struct ns_srv_reply *srv_lookup( char *service, char *protocol, char *domain )
 	if( ns_initparse( querybuf, size, &nsh ) != 0 )
 		return NULL;
 	
-	if( ns_parserr( &nsh, ns_s_an, 0, &rr ) != 0 )
-		return NULL;
-	
-	size = ns_rr_rdlen( rr );
-	buf = ns_rr_rdata( rr );
-	
-	len = 0;
-	for( i = 6; i < size && buf[i]; i += buf[i] + 1 )
-		len += buf[i] + 1;
-	
-	if( i > size )
-		return NULL;
-	
-	reply = g_malloc( sizeof( struct ns_srv_reply ) + len );
-	memcpy( reply->name, buf + 7, len );
-	
-	for( i = buf[6]; i < len && buf[7+i]; i += buf[7+i] + 1 )
-		reply->name[i] = '.';
-	
-	if( i > len )
+	n = 0;
+	while( ns_parserr( &nsh, ns_s_an, n, &rr ) == 0 )
 	{
-		g_free( reply );
-		return NULL;
+		size = ns_rr_rdlen( rr );
+		buf = ns_rr_rdata( rr );
+		
+		len = 0;
+		for( i = 6; i < size && buf[i]; i += buf[i] + 1 )
+			len += buf[i] + 1;
+		
+		if( i > size )
+			break;
+		
+		reply = g_malloc( sizeof( struct ns_srv_reply ) + len );
+		memcpy( reply->name, buf + 7, len );
+		
+		for( i = buf[6]; i < len && buf[7+i]; i += buf[7+i] + 1 )
+			reply->name[i] = '.';
+		
+		if( i > len )
+		{
+			g_free( reply );
+			break;
+		}
+		
+		reply->prio = ( buf[0] << 8 ) | buf[1];
+		reply->weight = ( buf[2] << 8 ) | buf[3];
+		reply->port = ( buf[4] << 8 ) | buf[5];
+		
+		n ++;
+		replies = g_renew( struct ns_srv_reply *, replies, n + 1 );
+		replies[n-1] = reply;
 	}
-	
-	reply->prio = ( buf[0] << 8 ) | buf[1];
-	reply->weight = ( buf[2] << 8 ) | buf[3];
-	reply->port = ( buf[4] << 8 ) | buf[5];
+	if( replies )
+		replies[n] = NULL;
 #endif
 	
-	return reply;
+	return replies;
+}
+
+void srv_free( struct ns_srv_reply **srv )
+{
+	int i;
+	
+	if( srv == NULL )
+		return;
+	
+	for( i = 0; srv[i]; i ++ )
+		g_free( srv[i] );
+	g_free( srv );
 }
 
 /* Word wrapping. Yes, I know this isn't UTF-8 clean. I'm willing to take the risk. */
@@ -651,6 +678,9 @@ int md5_verify_password( char *password, char *hash )
 	return ret;
 }
 
+/* Split commands (root-style, *not* IRC-style). Handles "quoting of"
+   white\ space in 'various ways'. Returns a NULL-terminated static
+   char** so watch out with nested use! Definitely not thread-safe. */
 char **split_command_parts( char *command )
 {
 	static char *cmd[IRC_MAX_ARGS+1];
