@@ -251,7 +251,7 @@ void imcb_error( struct im_connection *ic, char *format, ... )
 	if( ic->flags & OPT_LOGGED_IN )
 		serv_got_crap( ic, "Error: %s", text );
 	else
-		serv_got_crap( ic, "Couldn't log in: %s", text );
+		serv_got_crap( ic, "Login error: %s", text );
 	
 	g_free( text );
 }
@@ -276,6 +276,7 @@ void imcb_connected( struct im_connection *ic )
 	
 	imcb_log( ic, "Logged in" );
 	
+	b_event_remove( ic->keepalive );
 	ic->keepalive = b_timeout_add( 60000, send_keepalive, ic );
 	ic->flags |= OPT_LOGGED_IN;
 	
@@ -325,14 +326,6 @@ void imc_logout( struct im_connection *ic, int allow_reconnect )
 	
 	imcb_log( ic, "Signing off.." );
 	
-	b_event_remove( ic->keepalive );
-	ic->keepalive = 0;
-	ic->acc->prpl->logout( ic );
-	b_event_remove( ic->inpa );
-	
-	g_free( ic->away );
-	ic->away = NULL;
-	
 	for( l = bee->users; l; )
 	{
 		bee_user_t *bu = l->data;
@@ -344,6 +337,14 @@ void imc_logout( struct im_connection *ic, int allow_reconnect )
 		l = next;
 	}
 	
+	b_event_remove( ic->keepalive );
+	ic->keepalive = 0;
+	ic->acc->prpl->logout( ic );
+	b_event_remove( ic->inpa );
+	
+	g_free( ic->away );
+	ic->away = NULL;
+	
 	query_del_by_conn( (irc_t*) ic->bee->ui_data, ic );
 	
 	for( a = bee->accounts; a; a = a->next )
@@ -467,7 +468,7 @@ void imcb_ask_auth( struct im_connection *ic, const char *handle, const char *re
 		realname_ = g_strdup_printf( " (%s)", realname );
 	
 	s = g_strdup_printf( "The user %s%s wants to add you to his/her buddy list.",
-	                     handle, realname_ ?: "" );
+	                     handle, realname_ ? realname_ : "" );
 	
 	g_free( realname_ );
 	
@@ -490,7 +491,7 @@ static void imcb_ask_add_cb_yes( void *data )
 	
 	cbd->ic->acc->prpl->add_buddy( cbd->ic, cbd->handle, NULL );
 	
-	return imcb_ask_add_cb_no( data );
+	imcb_ask_add_cb_no( data );
 }
 
 void imcb_ask_add( struct im_connection *ic, const char *handle, const char *realname )
