@@ -368,16 +368,19 @@ static gboolean oscar_login_connect(gpointer data, gint source, b_input_conditio
 static void oscar_init(account_t *acc)
 {
 	set_t *s;
+	gboolean icq = isdigit(acc->user[0]);
 	
-	if (isdigit(acc->user[0])) {
+	if (icq) {
 		set_add(&acc->set, "ignore_auth_requests", "false", set_eval_bool, acc);
 		set_add(&acc->set, "old_icq_auth", "false", set_eval_bool, acc);
 	}
 	
-	s = set_add(&acc->set, "server", AIM_DEFAULT_LOGIN_SERVER, set_eval_account, acc);
+	s = set_add(&acc->set, "server",
+	            icq ? AIM_DEFAULT_LOGIN_SERVER_ICQ
+	                : AIM_DEFAULT_LOGIN_SERVER_AIM, set_eval_account, acc);
 	s->flags |= ACC_SET_NOSAVE | ACC_SET_OFFLINE_ONLY;
 	
-	if(isdigit(acc->user[0])) {
+	if (icq) {
 		s = set_add(&acc->set, "web_aware", "false", set_eval_bool, acc);
 		s->flags |= ACC_SET_OFFLINE_ONLY;
 	}
@@ -516,6 +519,14 @@ static int gaim_parse_auth_resp(aim_session_t *sess, aim_frame_t *fr, ...) {
 		case 0x05:
 			/* Incorrect nick/password */
 			imcb_error(ic, _("Incorrect nickname or password."));
+			{
+				int max = od->icq ? 8 : 16;
+				if (strlen(ic->acc->pass) > max)
+					imcb_log(ic, "Note that the maximum password "
+					         "length supported by this protocol is "
+					         "%d characters, try logging in using "
+					         "a shorter password.", max);
+			}
 //			plugin_event(event_error, (void *)980, 0, 0, 0);
 			break;
 		case 0x11:
@@ -898,6 +909,10 @@ static int gaim_parse_oncoming(aim_session_t *sess, aim_frame_t *fr, ...) {
 			flags |= OPT_AWAY;
 	}
 	
+	/* Maybe this should be done just for AIM contacts, not sure. */
+	if (info->flags & AIM_FLAG_WIRELESS)
+		flags |= OPT_MOBILE;
+	
 	if (info->present & AIM_USERINFO_PRESENT_ICQEXTSTATUS) {
 		if (!(info->icqinfo.status & AIM_ICQ_STATE_CHAT) &&
 		      (info->icqinfo.status != AIM_ICQ_STATE_NORMAL)) {
@@ -2599,6 +2614,7 @@ void oscar_initmodule()
 {
 	struct prpl *ret = g_new0(struct prpl, 1);
 	ret->name = "oscar";
+    ret->mms = 2343;       /* this guess taken from libotr UPGRADING file */
 	ret->away_states = oscar_away_states;
 	ret->init = oscar_init;
 	ret->login = oscar_login;
