@@ -37,11 +37,14 @@ typedef enum
 	TWITTER_HAVE_FRIENDS = 1,
 } twitter_flags_t;
 
+struct twitter_log_data;
+
 struct twitter_data
 {
 	char* user;
-	char* pass;
 	struct oauth_info *oauth_info;
+	GSList *follow_ids;
+	
 	guint64 home_timeline_id;
 	guint64 last_status_id; /* For undo */
 	gint main_loop_id;
@@ -49,12 +52,17 @@ struct twitter_data
 	gint http_fails;
 	twitter_flags_t flags;
 	
+	/* set base_url */
 	gboolean url_ssl;
 	int url_port;
 	char *url_host;
 	char *url_path;
 
 	char *prefix; /* Used to generate contact + channel name. */
+	
+	/* set show_ids */
+	struct twitter_log_data *log;
+	int log_id;
 };
 
 struct twitter_user_data
@@ -63,6 +71,13 @@ struct twitter_user_data
 	time_t last_time;
 };
 
+#define TWITTER_LOG_LENGTH 100
+struct twitter_log_data
+{
+	guint64 id;
+	struct bee_user *bu; /* DANGER: can be a dead pointer. Check it first. */
+};
+
 /**
  * This has the same function as the msn_connections GSList. We use this to 
  * make sure the connection is still alive in callbacks before we do anything
@@ -72,4 +87,7 @@ extern GSList *twitter_connections;
 
 void twitter_login_finish( struct im_connection *ic );
 
+struct http_request;
+char *twitter_parse_error( struct http_request *req );
+
 #endif //_TWITTER_H
