@@ -113,7 +113,10 @@ static void cmd_identify( irc_t *irc, char **cmd )
 		return;
 	}
 	
-	if( strncmp( cmd[1], "-no", 3 ) == 0 )
+	if( cmd[1] == NULL )
+	{
+	}
+	else if( strncmp( cmd[1], "-no", 3 ) == 0 )
 	{
 		load = FALSE;
 		password = cmd[2];
@@ -134,7 +137,9 @@ static void cmd_identify( irc_t *irc, char **cmd )
 	
 	if( password == NULL )
 	{
-		MIN_ARGS( 2 );
+		irc_usermsg( irc, "About to identify, use /OPER to enter the password" );
+		irc->status |= OPER_HACK_IDENTIFY;
+		return;
 	}
 	
 	if( load )
@@ -212,6 +217,13 @@ static void cmd_register( irc_t *irc, char **cmd )
 		irc_usermsg( irc, "This server does not allow registering new accounts" );
 		return;
 	}
+	
+	if( cmd[1] == NULL )
+	{
+		irc_usermsg( irc, "About to register, use /OPER to enter the password" );
+		irc->status |= OPER_HACK_REGISTER;
+		return;
+	}
 
 	switch( storage_save( irc, cmd[1], FALSE ) ) {
 		case STORAGE_ALREADY_EXISTS:
@@ -396,6 +408,7 @@ static void cmd_account( irc_t *irc, char **cmd )
 		MIN_ARGS( 3 );
 		
 		if( cmd[4] == NULL )
+		{
 			for( a = irc->b->accounts; a; a = a->next )
 				if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
 				{
@@ -403,6 +416,9 @@ static void cmd_account( irc_t *irc, char **cmd )
 					             "first (use /OPER)", a->prpl->name, a->user );
 					return;
 				}
+			
+			irc->status |= OPER_HACK_ACCOUNT_ADD;
+		}
 		
 		prpl = find_protocol( cmd[2] );
 		
@@ -773,13 +789,20 @@ static void cmd_info( irc_t *irc, char **cmd )
 static void cmd_rename( irc_t *irc, char **cmd )
 {
 	irc_user_t *iu, *old;
+	gboolean del = g_strcasecmp( cmd[1], "-del" ) == 0;
 	
-	iu = irc_user_by_name( irc, cmd[1] );
+	iu = irc_user_by_name( irc, cmd[del ? 2 : 1] );
 	
 	if( iu == NULL )
 	{
 		irc_usermsg( irc, "Nick `%s' does not exist", cmd[1] );
 	}
+	else if( del )
+	{
+		if( iu->bu )
+			bee_irc_user_nick_reset( iu );
+		irc_usermsg( irc, "Nickname reset to `%s'", iu->nick );
+	}
 	else if( iu == irc->user )
 	{
 		irc_usermsg( irc, "Use /nick to change your own nickname" );
@@ -1354,12 +1377,12 @@ command_t root_commands[] = {
 	{ "ft",             0, cmd_transfer,       0 },
 	{ "group",          1, cmd_group,          0 },
 	{ "help",           0, cmd_help,           0 }, 
-	{ "identify",       1, cmd_identify,       0 },
+	{ "identify",       0, cmd_identify,       0 },
 	{ "info",           1, cmd_info,           0 },
 	{ "nick",           1, cmd_nick,           0 },
 	{ "no",             0, cmd_yesno,          0 },
 	{ "qlist",          0, cmd_qlist,          0 },
-	{ "register",       1, cmd_register,       0 },
+	{ "register",       0, cmd_register,       0 },
 	{ "remove",         1, cmd_remove,         0 },
 	{ "rename",         2, cmd_rename,         0 },
 	{ "save",           0, cmd_save,           0 },
