@@ -319,6 +319,16 @@ static void xml_text( GMarkupParseContext *ctx, const gchar *text_orig, gsize te
 	}
 	else if( g_strcasecmp( g_markup_parse_context_get_element( ctx ), "setting" ) == 0 && xd->current_setting )
 	{
+		if( xd->current_account )
+		{
+			set_t *s = set_find( xd->current_set_head, xd->current_setting );
+			if( s && ( s->flags & ACC_SET_ONLINE_ONLY ) )
+			{
+				g_free( xd->current_setting );
+				xd->current_setting = NULL;
+				return;
+			}
+		}
 		set_setstr( xd->current_set_head, xd->current_setting, (char*) text );
 		g_free( xd->current_setting );
 		xd->current_setting = NULL;
@@ -483,7 +493,7 @@ static storage_status_t xml_save( irc_t *irc, int overwrite )
 	g_free( pass_buf );
 	
 	for( set = irc->b->set; set; set = set->next )
-		if( set->value )
+		if( set->value && !( set->flags & SET_NOSAVE ) )
 			if( !xml_printf( fd, 1, "<setting name=\"%s\">%s</setting>\n", set->key, set->value ) )
 				goto write_error;
 	
